/*
** iplog_udp.c - iplog UDP traffic logger.
** Copyright (C) 1999 behe <eric@ojnk.net>, Ryan McCabe <odin@numb.org>
** Copyright (C) 2000 Ryan McCabe <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
**
** $Id: iplog_udp.c,v 1.10 2000/02/10 22:01:01 odin Exp $
*/

#include <config.h>

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>

#include <pthread.h>

#include <iplog.h>
#include <iplog_filter.h>
#include <iplog_options.h>
#include <iplog_scan.h>

#ifndef HAVE_UH_SPORT
#	define udphdr my_udp_header
struct my_udp_header {
	u_int16_t uh_sport;
	u_int16_t uh_dport;
	u_int16_t uh_ulen;
	u_int16_t uh_sum;
};
#endif

extern struct filter_data *filters[3];
extern u_long flags;

static struct udp_data {
	addr_t t_host;
	time_t t_time;
} udp_data;

__inline__ bool udp_res(void) {
	return (flags & UDP_RES);
}

pthread_mutex_t udp_lock = PTHREAD_MUTEX_INITIALIZER;

int udp_parser(const struct ip *ip) {
	struct udphdr *udp = (struct udphdr *) ((char *) ip + (ip->ip_hl << 2));
	u_char buf[MAX_HSTLEN], sbuf[MAX_SRVLEN];
	u_long len;

	if (apply_filter(FIL_UDP, ip, udp->uh_sport, udp->uh_dport))
		return (0);

	len = ntohs(ip->ip_len) - (ip->ip_hl << 2);

	if ((flags & SMURF) && udp->uh_sport == htons(7) &&
		check_scan(SCAN_SMURF, ip, len) != 0)
	{
		return (0);
	}

	if ((flags & TRACEROUTE) && ip->ip_ttl == 1 && ntohs(udp->uh_dport) > 30000)
	{
		pthread_mutex_lock(&udp_lock);
		if (udp_data.t_host != ip->ip_src.s_addr ||
			(time(NULL) >= (udp_data.t_time + 15)))
		{
			udp_data.t_host = ip->ip_src.s_addr;
			time(&udp_data.t_time);

			pthread_mutex_unlock(&udp_lock);

			host_lookup(&ip->ip_src, udp_res(), buf, sizeof(buf));

			if (flags & LOG_DEST) {
				u_char buf2[MAX_HSTLEN];

				host_lookup(&ip->ip_dst, udp_res(), buf2, sizeof(buf2));
				mysyslog("UDP: traceroute from %s to %s", buf, buf2);
			} else {
				mysyslog("UDP: traceroute from %s", buf);
			}
		} else
			pthread_mutex_unlock(&udp_lock);

		return (0);
	}

	if ((flags & UDP_SCAN) && check_scan(SCAN_UDP, ip, len) != 0)
		return (0);

	if (flags & SCANS_ONLY)
		return (0);

	serv_lookup(udp->uh_dport, "udp", sbuf, sizeof(sbuf));
	host_lookup(&ip->ip_src, udp_res(), buf, sizeof(buf));

	if (flags & LOG_DEST) {
		u_char buf2[MAX_HSTLEN];

		host_lookup(&ip->ip_dst, udp_res(), buf2, sizeof(buf2));
		mysyslog("UDP: dgram to %s:%s from %s:%u (%lu data bytes)",
			buf2, sbuf, buf, ntohs(udp->uh_sport),
			ntohs(udp->uh_ulen) - sizeof(*udp));
	} else {
		mysyslog("UDP: dgram to %s from %s:%u (%lu data bytes)",
			sbuf, buf, ntohs(udp->uh_sport),
			ntohs(udp->uh_ulen) - sizeof(*udp));
	}

	return (0);
}

/* vim:ts=4:sw=8:tw=0 */
